
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>

#include <unistd.h>

int stdout_is_a_pipe() {
  struct stat s;
  fstat(1, &s);
  return !S_ISFIFO(s.st_mode);
}

#ifdef _WIN32
#include <windows.h>
#include <io.h>
#endif

int create_excl(const char *p)
{
    return open(p, O_EXCL | O_CREAT, 0666);
}

#ifdef _WIN32
int mkstemp(char *p)
{
    char* p2;
    int fd;
    int len = strlen(p);
    if (len < 6)
        return -1;
    snprintf(p+len-6, 6, "%06d", rand()+rand()<<16);
    fd = open(p, O_CREAT | O_EXCL | O_RDWR, 0666);
    return fd;
}

int pipe( int fildes[2] ) {
return _pipe( fildes, 8 * (2<<10), O_BINARY );
}

int isnt_symlink(const char *file) {
  return 1; /* FIXME: should ignore windows shortcuts */
}
#else

int open_read(const char *fname) {
  return open(fname, O_RDONLY);
}

int open_write(const char *fname) {
  return open(fname, O_WRONLY | O_TRUNC | O_CREAT,
              S_IWUSR | S_IRUSR | S_IROTH | S_IRGRP);
}

#include <sys/wait.h>
#include <signal.h>

/* This waits and then returns the exit status of the process that was
   waited for. */
int smart_wait(int pid) {
  int stat;
  waitpid(pid, &stat, 0);
  if (WIFEXITED(stat)) {
    return WEXITSTATUS(stat);
  } else if (WIFSIGNALED(stat)) {
    psignal(WTERMSIG(stat), "Error in subprocess");
    return - WTERMSIG(stat);
  } else {
    return -137;
  }
}

#include <unistd.h>
#include <sys/time.h>

int execvp_no_vtalarm(const char *file, char *const argv[]) {
  /* Reset the itimers in the child, so it doesn't get plagued
   * by SIGVTALRM interrupts.
   */
  struct timeval tv_null = { 0, 0 };
  struct itimerval itv;
  itv.it_interval = tv_null;
  itv.it_value = tv_null;
  setitimer(ITIMER_REAL, &itv, NULL);
  setitimer(ITIMER_VIRTUAL, &itv, NULL);
  setitimer(ITIMER_PROF, &itv, NULL);
  return execvp(file, argv);
}

#include <sys/types.h>
#include <sys/stat.h>

int isnt_symlink(const char *file) {
  struct stat buf;
  if (lstat(file, &buf)) return 0; /* treat error as symlink */
  return !S_ISLNK(buf.st_mode);
}

#endif
